/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
module dwt.graphics.GCData;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.internal.cocoa.NSAffineTransform;
import dwt.internal.cocoa.NSBezierPath;
import dwt.internal.cocoa.NSRect;
import dwt.internal.cocoa.NSSize;
import dwt.internal.cocoa.NSView;

/**
 * Instances of this class are descriptions of GCs in terms
 * of unallocated platform-specific data fields.
 * <p>
 * <b>IMPORTANT:</b> This class is <em>not</em> part of the public
 * API for DWT. It is marked public only so that it can be shared
 * within the packages provided by DWT. It is not available on all
 * platforms, and should never be called from application code.
 * </p>
 */
public final class GCData {
    public Device device;
    public int style, state = -1;
    public float[] foreground;
    public float[] background;
    public Pattern foregroundPattern;
    public Pattern backgroundPattern;
    public Font font;
    public int alpha = 0xFF;
    public float lineWidth;
    public int lineStyle = DWT.LINE_SOLID;
    public int lineCap = DWT.CAP_FLAT;
    public int lineJoin = DWT.JOIN_MITER;
    public float lineDashesOffset;
    public float[] lineDashes;
    public float lineMiterLimit = 10;
    public bool xorMode;
    public int antialias = DWT.DEFAULT;
    public int textAntialias = DWT.DEFAULT;
    public int fillRule = DWT.FILL_EVEN_ODD;
    public int bitmapDataAddress;

    public Image image;
    
    public float drawXOffset, drawYOffset;
    public NSRect paintRect;
    public NSBezierPath path;
    public NSAffineTransform transform, inverseTransform;
    public NSBezierPath clipPath;
    public NSView view;
    public NSSize size;
}
