/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSComparisonPredicate;

import dwt.internal.cocoa.NSExpression;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSPredicate;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSComparisonPredicateModifier
{
    NSDirectPredicateModifier = 0,
    NSAllPredicateModifier,
    NSAnyPredicateModifier,
}

alias NSComparisonPredicateModifier.NSDirectPredicateModifier NSDirectPredicateModifier;
alias NSComparisonPredicateModifier.NSAllPredicateModifier NSAllPredicateModifier;
alias NSComparisonPredicateModifier.NSAnyPredicateModifier NSAnyPredicateModifier;

public class NSComparisonPredicate : NSPredicate
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public objc.id comparisonPredicateModifier ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_comparisonPredicateModifier);
    }

    public objc.SEL customSelector ()
    {
        return cast(objc.SEL) OS.objc_msgSend(this.id_, OS.sel_customSelector);
    }

    public NSComparisonPredicate initWithLeftExpression_rightExpression_customSelector_ (NSExpression lhs, NSExpression rhs, objc.SEL selector)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithLeftExpression_1rightExpression_1customSelector_1, lhs !is null ? lhs.id_ : null,
                rhs !is null ? rhs.id_ : null, selector);
        return result !is null ? this : null;
    }

    public NSComparisonPredicate initWithLeftExpression_rightExpression_modifier_type_options_ (NSExpression lhs, NSExpression rhs, objc.id modifier,
            objc.id type, NSUInteger options)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithLeftExpression_1rightExpression_1modifier_1type_1options_1,
                lhs !is null ? lhs.id_ : null, rhs !is null ? rhs.id_ : null, modifier, type, options);
        return result !is null ? this : null;
    }

    public NSExpression leftExpression ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_leftExpression);
        return result !is null ? new NSExpression(result) : null;
    }

    public NSUInteger options ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id_, OS.sel_options);
    }

    public objc.id predicateOperatorType ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_predicateOperatorType);
    }

    public static NSPredicate static_predicateWithLeftExpression_rightExpression_customSelector_ (NSExpression lhs, NSExpression rhs,
            objc.SEL selector)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSComparisonPredicate, OS.sel_predicateWithLeftExpression_1rightExpression_1customSelector_1,
                lhs !is null ? lhs.id_ : null, rhs !is null ? rhs.id_ : null, selector);
        return result !is null ? new NSPredicate(result) : null;
    }

    public static NSPredicate static_predicateWithLeftExpression_rightExpression_modifier_type_options_ (NSExpression lhs, NSExpression rhs,
            objc.id modifier, objc.id type, NSUInteger options)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSComparisonPredicate,
                OS.sel_predicateWithLeftExpression_1rightExpression_1modifier_1type_1options_1, lhs !is null ? lhs.id_ : null,
                rhs !is null ? rhs.id_ : null, modifier, type, options);
        return result !is null ? new NSPredicate(result) : null;
    }

    public NSExpression rightExpression ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_rightExpression);
        return result !is null ? new NSExpression(result) : null;
    }

}
