/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSImageCell;

import dwt.internal.cocoa.NSCell;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSImageAlignment
{
    NSImageAlignCenter = 0,
    NSImageAlignTop,
    NSImageAlignTopLeft,
    NSImageAlignTopRight,
    NSImageAlignLeft,
    NSImageAlignBottom,
    NSImageAlignBottomLeft,
    NSImageAlignBottomRight,
    NSImageAlignRight
}

enum NSImageFrameStyle
{
    NSImageFrameNone = 0,
    NSImageFramePhoto,
    NSImageFrameGrayBezel,
    NSImageFrameGroove,
    NSImageFrameButton
}

enum NSImageScaling
{
    NSScaleProportionally = 0,
    NSScaleToFit,
    NSScaleNone
}

public class NSImageCell : NSCell
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSImageAlignment imageAlignment ()
    {
        return cast(NSImageAlignment) OS.objc_msgSend(this.id_, OS.sel_imageAlignment);
    }

    public NSImageFrameStyle imageFrameStyle ()
    {
        return cast(NSImageFrameStyle) OS.objc_msgSend(this.id_, OS.sel_imageFrameStyle);
    }

    public NSImageScaling imageScaling ()
    {
        return cast(NSImageScaling) OS.objc_msgSend(this.id_, OS.sel_imageScaling);
    }

    public void setImageAlignment (NSImageAlignment newAlign)
    {
        OS.objc_msgSend(this.id_, OS.sel_setImageAlignment_1, newAlign);
    }

    public void setImageFrameStyle (NSImageFrameStyle newStyle)
    {
        OS.objc_msgSend(this.id_, OS.sel_setImageFrameStyle_1, newStyle);
    }

    public void setImageScaling (NSImageScaling newScaling)
    {
        OS.objc_msgSend(this.id_, OS.sel_setImageScaling_1, newScaling);
    }

}
