/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSLevelIndicatorCell;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSActionCell;
import dwt.internal.cocoa.NSImage;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSRect;
import dwt.internal.cocoa.NSSliderCell : NSTickMarkPosition;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

alias NSUInteger NSLevelIndicatorStyle;;

public class NSLevelIndicatorCell : NSActionCell
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public double criticalValue ()
    {
        return OS.objc_msgSend_fpret(this.id_, OS.sel_criticalValue);
    }

    public id initWithLevelIndicatorStyle (NSLevelIndicatorStyle levelIndicatorStyle)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithLevelIndicatorStyle_1, levelIndicatorStyle);
        return result !is null ? new id(result) : null;
    }

    public NSLevelIndicatorStyle levelIndicatorStyle ()
    {
        return cast(NSLevelIndicatorStyle) OS.objc_msgSend(this.id_, OS.sel_levelIndicatorStyle);
    }

    public double maxValue ()
    {
        return cast(double) OS.objc_msgSend_fpret(this.id_, OS.sel_maxValue);
    }

    public double minValue ()
    {
        return cast(double) OS.objc_msgSend_fpret(this.id_, OS.sel_minValue);
    }

    public NSInteger numberOfMajorTickMarks ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_numberOfMajorTickMarks);
    }

    public NSInteger numberOfTickMarks ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_numberOfTickMarks);
    }

    public NSRect rectOfTickMarkAtIndex (NSInteger index)
    {
        NSRect result;
        OS.objc_msgSend_stret(&result, this.id_, OS.sel_rectOfTickMarkAtIndex_1, index);
        return result;
    }

    public void setCriticalValue (double criticalValue)
    {
        OS.objc_msgSend(this.id_, OS.sel_setCriticalValue_1, criticalValue);
    }

    public void setImage (NSImage image)
    {
        OS.objc_msgSend(this.id_, OS.sel_setImage_1, image !is null ? image.id_ : null);
    }

    public void setLevelIndicatorStyle (NSLevelIndicatorStyle levelIndicatorStyle)
    {
        OS.objc_msgSend(this.id_, OS.sel_setLevelIndicatorStyle_1, levelIndicatorStyle);
    }

    public void setMaxValue (double maxValue)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMaxValue_1, maxValue);
    }

    public void setMinValue (double minValue)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMinValue_1, minValue);
    }

    public void setNumberOfMajorTickMarks (NSInteger count)
    {
        OS.objc_msgSend(this.id_, OS.sel_setNumberOfMajorTickMarks_1, count);
    }

    public void setNumberOfTickMarks (NSInteger count)
    {
        OS.objc_msgSend(this.id_, OS.sel_setNumberOfTickMarks_1, count);
    }

    public void setTickMarkPosition (NSTickMarkPosition position)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTickMarkPosition_1, position);
    }

    public void setWarningValue (double warningValue)
    {
        OS.objc_msgSend(this.id_, OS.sel_setWarningValue_1, warningValue);
    }

    public NSTickMarkPosition tickMarkPosition ()
    {
        return cast(NSTickMarkPosition) OS.objc_msgSend(this.id_, OS.sel_tickMarkPosition);
    }

    public double tickMarkValueAtIndex (NSInteger index)
    {
        return cast(double) OS.objc_msgSend_fpret(this.id_, OS.sel_tickMarkValueAtIndex_1, index);
    }

    public double warningValue ()
    {
        return cast(double) OS.objc_msgSend_fpret(this.id_, OS.sel_warningValue);
    }

}
