/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSMutableParagraphStyle;

import dwt.internal.cocoa.CGFloat;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSParagraphStyle;
import dwt.internal.cocoa.NSText : NSTextAlignment, NSWritingDirection;
import dwt.internal.cocoa.NSTextTab;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSLineBreakMode
{
    NSLineBreakByWordWrapping = 0,
    NSLineBreakByCharWrapping,
    NSLineBreakByClipping,
    NSLineBreakByTruncatingHead,
    NSLineBreakByTruncatingTail,
    NSLineBreakByTruncatingMiddle
}

alias NSLineBreakMode.NSLineBreakByWordWrapping NSLineBreakByWordWrapping;
alias NSLineBreakMode.NSLineBreakByCharWrapping NSLineBreakByCharWrapping;
alias NSLineBreakMode.NSLineBreakByClipping NSLineBreakByClipping;
alias NSLineBreakMode.NSLineBreakByTruncatingHead NSLineBreakByTruncatingHead;
alias NSLineBreakMode.NSLineBreakByTruncatingTail NSLineBreakByTruncatingTail;
alias NSLineBreakMode.NSLineBreakByTruncatingMiddle NSLineBreakByTruncatingMiddle;

public class NSMutableParagraphStyle : NSParagraphStyle
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void addTabStop (NSTextTab anObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_addTabStop_1, anObject !is null ? anObject.id_ : null);
    }

    public void removeTabStop (NSTextTab anObject)
    {
        OS.objc_msgSend(this.id_, OS.sel_removeTabStop_1, anObject !is null ? anObject.id_ : null);
    }

    public void setAlignment (NSTextAlignment alignment)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAlignment_1, alignment);
    }

    public void setBaseWritingDirection (NSWritingDirection writingDirection)
    {
        OS.objc_msgSend(this.id_, OS.sel_setBaseWritingDirection_1, writingDirection);
    }

    public void setDefaultTabInterval (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setDefaultTabInterval_1, aFloat);
    }

    public void setFirstLineHeadIndent (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setFirstLineHeadIndent_1, aFloat);
    }

    public void setHeadIndent (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setHeadIndent_1, aFloat);
    }

    public void setHeaderLevel (NSInteger level)
    {
        OS.objc_msgSend(this.id_, OS.sel_setHeaderLevel_1, level);
    }

    public void setHyphenationFactor (float aFactor)
    {
        OS.objc_msgSend(this.id_, OS.sel_setHyphenationFactor_1, aFactor);
    }

    public void setLineBreakMode (NSLineBreakMode mode)
    {
        OS.objc_msgSend(this.id_, OS.sel_setLineBreakMode_1, mode);
    }

    public void setLineHeightMultiple (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setLineHeightMultiple_1, aFloat);
    }

    public void setLineSpacing (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setLineSpacing_1, aFloat);
    }

    public void setMaximumLineHeight (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMaximumLineHeight_1, aFloat);
    }

    public void setMinimumLineHeight (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setMinimumLineHeight_1, aFloat);
    }

    public void setParagraphSpacing (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setParagraphSpacing_1, aFloat);
    }

    public void setParagraphSpacingBefore (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setParagraphSpacingBefore_1, aFloat);
    }

    public void setParagraphStyle (NSParagraphStyle obj)
    {
        OS.objc_msgSend(this.id_, OS.sel_setParagraphStyle_1, obj !is null ? obj.id_ : null);
    }

    public void setTabStops (NSArray array)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTabStops_1, array !is null ? array.id_ : null);
    }

    public void setTailIndent (CGFloat aFloat)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTailIndent_1, aFloat);
    }

    public void setTextBlocks (NSArray array)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTextBlocks_1, array !is null ? array.id_ : null);
    }

    public void setTextLists (NSArray array)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTextLists_1, array !is null ? array.id_ : null);
    }

    public void setTighteningFactorForTruncation (float aFactor)
    {
        OS.objc_msgSend(this.id_, OS.sel_setTighteningFactorForTruncation_1, aFactor);
    }

}
