/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSPositionalSpecifier;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSScriptClassDescription;
import dwt.internal.cocoa.NSScriptObjectSpecifier;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSInsertionPosition
{
    NSPositionAfter,
    NSPositionBefore,
    NSPositionBeginning,
    NSPositionEnd,
    NSPositionReplace
}

alias NSInsertionPosition.NSPositionAfter NSPositionAfter;
alias NSInsertionPosition.NSPositionBefore NSPositionBefore;
alias NSInsertionPosition.NSPositionBeginning NSPositionBeginning;
alias NSInsertionPosition.NSPositionEnd NSPositionEnd;
alias NSInsertionPosition.NSPositionReplace NSPositionReplace;

public class NSPositionalSpecifier : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void evaluate ()
    {
        OS.objc_msgSend(this.id_, OS.sel_evaluate);
    }

    public NSPositionalSpecifier initWithPosition (NSInsertionPosition position, NSScriptObjectSpecifier specifier)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithPosition_1objectSpecifier_1, position, specifier !is null ? specifier.id_ : null);
        return result !is null ? this : null;
    }

    public id insertionContainer ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_insertionContainer);
        return result !is null ? new id(result) : null;
    }

    public NSInteger insertionIndex ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id_, OS.sel_insertionIndex);
    }

    public NSString insertionKey ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_insertionKey);
        return result !is null ? new NSString(result) : null;
    }

    public bool insertionReplaces ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_insertionReplaces) !is null;
    }

    public NSScriptObjectSpecifier objectSpecifier ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_objectSpecifier);
        return result !is null ? new NSScriptObjectSpecifier(result) : null;
    }

    public NSInsertionPosition position ()
    {
        return cast(NSInsertionPosition) OS.objc_msgSend(this.id_, OS.sel_position);
    }

    public void setInsertionClassDescription (NSScriptClassDescription classDescription)
    {
        OS.objc_msgSend(this.id_, OS.sel_setInsertionClassDescription_1, classDescription !is null ? classDescription.id_ : null);
    }

}
