/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSPredicateEditorRowTemplate;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSAttributeType;
import dwt.internal.cocoa.NSComparisonPredicate : NSComparisonPredicateModifier;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSPredicate;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSPredicateEditorRowTemplate : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSArray compoundTypes ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_compoundTypes);
        return result !is null ? new NSArray(result) : null;
    }

    public NSArray displayableSubpredicatesOfPredicate (NSPredicate predicate)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_displayableSubpredicatesOfPredicate_1, predicate !is null ? predicate.id_ : null);
        return result !is null ? new NSArray(result) : null;
    }

    public id initWithCompoundTypes (NSArray compoundTypes)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithCompoundTypes_1, compoundTypes !is null ? compoundTypes.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithLeftExpressions_rightExpressionAttributeType_modifier_operators_options_ (NSArray leftExpressions,
            NSAttributeType attributeType, NSComparisonPredicateModifier modifier, NSArray operators, NSUInteger options)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithLeftExpressions_1rightExpressionAttributeType_1modifier_1operators_1options_1,
                leftExpressions !is null ? leftExpressions.id_ : null, attributeType, modifier, operators !is null ? operators.id_ : null, options);
        return result !is null ? new id(result) : null;
    }

    public id initWithLeftExpressions_rightExpressions_modifier_operators_options_ (NSArray leftExpressions, NSArray rightExpressions, NSComparisonPredicateModifier modifier,
            NSArray operators, NSUInteger options)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithLeftExpressions_1rightExpressions_1modifier_1operators_1options_1,
                leftExpressions !is null ? leftExpressions.id_ : null, rightExpressions !is null ? rightExpressions.id_ : null, modifier,
                operators !is null ? operators.id_ : null, options);
        return result !is null ? new id(result) : null;
    }

    public NSArray leftExpressions ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_leftExpressions);
        return result !is null ? new NSArray(result) : null;
    }

    public double matchForPredicate (NSPredicate predicate)
    {
        return OS.objc_msgSend_fpret(this.id_, OS.sel_matchForPredicate_1, predicate !is null ? predicate.id_ : null);
    }

    public NSComparisonPredicateModifier modifier ()
    {
        return cast(NSComparisonPredicateModifier) OS.objc_msgSend(this.id_, OS.sel_modifier);
    }

    public NSArray operators ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_operators);
        return result !is null ? new NSArray(result) : null;
    }

    public NSUInteger options ()
    {
        return cast(NSUInteger) OS.objc_msgSend(this.id_, OS.sel_options);
    }

    public NSPredicate predicateWithSubpredicates (NSArray subpredicates)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_predicateWithSubpredicates_1, subpredicates !is null ? subpredicates.id_ : null);
        return result !is null ? new NSPredicate(result) : null;
    }

    public NSAttributeType rightExpressionAttributeType ()
    {
        return cast(NSAttributeType) OS.objc_msgSend(this.id_, OS.sel_rightExpressionAttributeType);
    }

    public NSArray rightExpressions ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_rightExpressions);
        return result !is null ? new NSArray(result) : null;
    }

    public void setPredicate (NSPredicate predicate)
    {
        OS.objc_msgSend(this.id_, OS.sel_setPredicate_1, predicate !is null ? predicate.id_ : null);
    }

    public NSArray templateViews ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_templateViews);
        return result !is null ? new NSArray(result) : null;
    }

//public static NSArray templatesWithAttributeKeyPaths(NSArray keyPaths, NSEntityDescription entityDescription) {
//  objc.id result = OS.objc_msgSend(OS.class_NSPredicateEditorRowTemplate, OS.sel_templatesWithAttributeKeyPaths_1inEntityDescription_1, keyPaths !is null ? keyPaths.id_ : null, entityDescription !is null ? entityDescription.id_ : null);
//  return result !is null ? new NSArray(result) : null;
//}

}
