/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSTextTab;

import dwt.internal.cocoa.CGFloat;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSText : NSTextAlignment;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

enum NSTextTabType
{
    NSLeftTabStopType = 0,
    NSRightTabStopType,
    NSCenterTabStopType,
    NSDecimalTabStopType
}

alias NSTextTabType.NSLeftTabStopType NSLeftTabStopType;
alias NSTextTabType.NSRightTabStopType NSRightTabStopType;
alias NSTextTabType.NSCenterTabStopType NSCenterTabStopType;
alias NSTextTabType.NSDecimalTabStopType NSDecimalTabStopType;

public class NSTextTab : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSTextAlignment alignment ()
    {
        return cast(NSTextAlignment) OS.objc_msgSend(this.id_, OS.sel_alignment);
    }

    public NSTextTab initWithTextAlignment (NSTextAlignment alignment, CGFloat loc, NSDictionary options)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithTextAlignment_1location_1options_1, alignment, loc,
                options !is null ? options.id_ : null);
        return result !is null ? this : null;
    }

    public NSTextTab initWithType (NSTextTabType type, CGFloat loc)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithType_1location_1, type, loc);
        return result !is null ? this : null;
    }

    public CGFloat location ()
    {
        return cast(CGFloat) OS.objc_msgSend_fpret(this.id_, OS.sel_location);
    }

    public NSDictionary options ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_options);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSTextTabType tabStopType ()
    {
        return cast(NSTextTabType) OS.objc_msgSend(this.id_, OS.sel_tabStopType);
    }

}
