/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSUserDefaultsController;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSController;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSUserDefaults;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSUserDefaultsController : NSController
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public bool appliesImmediately ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_appliesImmediately) !is null;
    }

    public NSUserDefaults defaults ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_defaults);
        return result !is null ? new NSUserDefaults(result) : null;
    }

    public bool hasUnappliedChanges ()
    {
        return OS.objc_msgSend(this.id_, OS.sel_hasUnappliedChanges) !is null;
    }

    public id initWithDefaults (NSUserDefaults defaults, NSDictionary initialValues)
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initWithDefaults_1initialValues_1, defaults !is null ? defaults.id_ : null,
                initialValues !is null ? initialValues.id_ : null);
        return result !is null ? new id(result) : null;
    }

    public NSDictionary initialValues ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_initialValues);
        return result !is null ? new NSDictionary(result) : null;
    }

    public void revert (id sender)
    {
        OS.objc_msgSend(this.id_, OS.sel_revert_1, sender !is null ? sender.id_ : null);
    }

    public void revertToInitialValues (id sender)
    {
        OS.objc_msgSend(this.id_, OS.sel_revertToInitialValues_1, sender !is null ? sender.id_ : null);
    }

    public void save (id sender)
    {
        OS.objc_msgSend(this.id_, OS.sel_save_1, sender !is null ? sender.id_ : null);
    }

    public void setAppliesImmediately (bool flag)
    {
        OS.objc_msgSend(this.id_, OS.sel_setAppliesImmediately_1, flag);
    }

    public void setInitialValues (NSDictionary initialValues)
    {
        OS.objc_msgSend(this.id_, OS.sel_setInitialValues_1, initialValues !is null ? initialValues.id_ : null);
    }

    public static id sharedUserDefaultsController ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSUserDefaultsController, OS.sel_sharedUserDefaultsController);
        return result !is null ? new id(result) : null;
    }

    public id values ()
    {
        objc.id result = OS.objc_msgSend(this.id_, OS.sel_values);
        return result !is null ? new id(result) : null;
    }

}
